@@ -41,6 +41,10 @@
 
 #[cfg(feature = "serde")]
 extern crate serde;
+
+#[cfg(test)]
+extern crate bincode;
+
 pub extern crate typenum;
 
 mod hex;
@@ -49,23 +53,23 @@ mod impls;
 #[cfg(feature = "serde")]
 pub mod impl_serde;
 
-use core::{mem, ptr, slice};
 use core::iter::FromIterator;
 use core::marker::PhantomData;
 use core::mem::ManuallyDrop;
 use core::ops::{Deref, DerefMut};
+use core::{mem, ptr, slice};
 use typenum::bit::{B0, B1};
 use typenum::uint::{UInt, UTerm, Unsigned};
 
 #[cfg_attr(test, macro_use)]
 pub mod arr;
+pub mod functional;
 pub mod iter;
 pub mod sequence;
-pub mod functional;
 
-use functional::*;
-pub use iter::GenericArrayIter;
-use sequence::*;
+use self::functional::*;
+pub use self::iter::GenericArrayIter;
+use self::sequence::*;
 
 /// Trait making `GenericArray` work, marking types to be used as length of an array
 pub unsafe trait ArrayLength<T>: Unsigned {
@@ -94,9 +98,9 @@ impl<T: Clone, U: Clone> Clone for GenericArrayImplEven<T, U> {
             parent1: self.parent1.clone(),
             parent2: self.parent2.clone(),
             _marker: PhantomData,
-        }
     }
 }
+}
 
 impl<T: Copy, U: Copy> Copy for GenericArrayImplEven<T, U> {}
 
@@ -116,9 +120,9 @@ impl<T: Clone, U: Clone> Clone for GenericArrayImplOdd<T, U> {
             parent1: self.parent1.clone(),
             parent2: self.parent2.clone(),
             data: self.data.clone(),
-        }
     }
 }
+}
 
 impl<T: Copy, U: Copy> Copy for GenericArrayImplOdd<T, U> {}
 
@@ -161,21 +165,43 @@ where
     }
 }
 
-struct ArrayBuilder<T, N: ArrayLength<T>> {
+/// Creates an array one element at a time using a mutable iterator
+/// you can write to with `ptr::write`.
+///
+/// Incremenent the position while iterating to mark off created elements,
+/// which will be dropped if `into_inner` is not called.
+#[doc(hidden)]
+pub struct ArrayBuilder<T, N: ArrayLength<T>> {
     array: ManuallyDrop<GenericArray<T, N>>,
     position: usize,
 }
 
 impl<T, N: ArrayLength<T>> ArrayBuilder<T, N> {
-    fn new() -> ArrayBuilder<T, N> {
+    #[doc(hidden)]
+    #[inline]
+    pub unsafe fn new() -> ArrayBuilder<T, N> {
         ArrayBuilder {
-            array: ManuallyDrop::new(unsafe { mem::uninitialized() }),
+            array: ManuallyDrop::new(mem::uninitialized()),
             position: 0,
         }
     }
 
-    fn into_inner(self) -> GenericArray<T, N> {
-        let array = unsafe { ptr::read(&self.array) };
+    /// Creates a mutable iterator for writing to the array using `ptr::write`.
+    ///
+    /// Increment the position value given as a mutable reference as you iterate
+    /// to mark how many elements have been created.
+    #[doc(hidden)]
+    #[inline]
+    pub unsafe fn iter_position(&mut self) -> (slice::IterMut<T>, &mut usize) {
+        (self.array.iter_mut(), &mut self.position)
+    }
+
+    /// When done writing (assuming all elements have been written to),
+    /// get the inner array.
+    #[doc(hidden)]
+    #[inline]
+    pub unsafe fn into_inner(self) -> GenericArray<T, N> {
+        let array = ptr::read(&self.array);
 
         mem::forget(self);
 
@@ -185,7 +211,7 @@ impl<T, N: ArrayLength<T>> ArrayBuilder<T, N> {
 
 impl<T, N: ArrayLength<T>> Drop for ArrayBuilder<T, N> {
     fn drop(&mut self) {
-        for value in self.array.iter_mut().take(self.position) {
+        for value in &mut self.array[..self.position] {
             unsafe {
                 ptr::drop_in_place(value);
             }
@@ -193,25 +219,42 @@ impl<T, N: ArrayLength<T>> Drop for ArrayBuilder<T, N> {
     }
 }
 
-struct ArrayConsumer<T, N: ArrayLength<T>> {
+/// Consumes an array.
+///
+/// Increment the position while iterating and any leftover elements
+/// will be dropped if position does not go to N
+#[doc(hidden)]
+pub struct ArrayConsumer<T, N: ArrayLength<T>> {
     array: ManuallyDrop<GenericArray<T, N>>,
     position: usize,
 }
 
 impl<T, N: ArrayLength<T>> ArrayConsumer<T, N> {
-    fn new(array: GenericArray<T, N>) -> ArrayConsumer<T, N> {
+    #[doc(hidden)]
+    #[inline]
+    pub unsafe fn new(array: GenericArray<T, N>) -> ArrayConsumer<T, N> {
         ArrayConsumer {
             array: ManuallyDrop::new(array),
             position: 0,
         }
     }
+
+    /// Creates an iterator and mutable reference to the internal position
+    /// to keep track of consumed elements.
+    ///
+    /// Increment the position as you iterate to mark off consumed elements
+    #[doc(hidden)]
+    #[inline]
+    pub unsafe fn iter_position(&mut self) -> (slice::Iter<T>, &mut usize) {
+        (self.array.iter(), &mut self.position)
+    }
 }
 
 impl<T, N: ArrayLength<T>> Drop for ArrayConsumer<T, N> {
     fn drop(&mut self) {
-        for i in self.position..N::to_usize() {
+        for value in &mut self.array[self.position..N::to_usize()] {
             unsafe {
-                ptr::drop_in_place(self.array.get_unchecked_mut(i));
+                ptr::drop_in_place(value);
             }
         }
     }
@@ -249,21 +292,27 @@ where
     where
         I: IntoIterator<Item = T>,
     {
-        let mut destination = ArrayBuilder::new();
+        unsafe {
+            let mut destination = ArrayBuilder::new();
 
-        for (src, dst) in iter.into_iter().zip(destination.array.iter_mut()) {
-            unsafe {
-                ptr::write(dst, src);
+            {
+                let (destination_iter, position) = destination.iter_position();
+
+                iter.into_iter()
+                    .zip(destination_iter)
+                    .for_each(|(src, dst)| {
+                        ptr::write(dst, src);
+
+                        *position += 1;
+                    });
             }
 
-            destination.position += 1;
-        }
+            if destination.position < N::to_usize() {
+                from_iter_length_fail(destination.position, N::to_usize());
+            }
 
-        if destination.position < N::to_usize() {
-            from_iter_length_fail(destination.position, N::to_usize());
+            destination.into_inner()
         }
-
-        destination.into_inner()
     }
 }
 
@@ -288,17 +337,21 @@ where
     where
         F: FnMut(usize) -> T,
     {
-        let mut destination = ArrayBuilder::new();
+        unsafe {
+            let mut destination = ArrayBuilder::new();
 
-        for (i, dst) in destination.array.iter_mut().enumerate() {
-            unsafe {
-                ptr::write(dst, f(i));
+            {
+                let (destination_iter, position) = destination.iter_position();
+
+                destination_iter.enumerate().for_each(|(i, dst)| {
+                    ptr::write(dst, f(i));
+
+                    *position += 1;
+                });
             }
 
-            destination.position += 1;
+            destination.into_inner()
         }
-
-        destination.into_inner()
     }
 
     #[doc(hidden)]
@@ -308,33 +361,29 @@ where
         mut f: F,
     ) -> MappedSequence<GenericArray<B, Self::Length>, B, U>
     where
-        GenericArray<B, Self::Length>: GenericSequence<B, Length = Self::Length>
-            + MappedGenericSequence<B, U>,
+        GenericArray<B, Self::Length>:
+            GenericSequence<B, Length = Self::Length> + MappedGenericSequence<B, U>,
         Self: MappedGenericSequence<T, U>,
         Self::Length: ArrayLength<B> + ArrayLength<U>,
         F: FnMut(B, Self::Item) -> U,
     {
-        let mut left = ArrayConsumer::new(lhs);
-        let mut right = ArrayConsumer::new(self);
+        unsafe {
+            let mut left = ArrayConsumer::new(lhs);
+            let mut right = ArrayConsumer::new(self);
 
-        let ArrayConsumer {
-            array: ref left_array,
-            position: ref mut left_position,
-        } = left;
-        let ArrayConsumer {
-            array: ref right_array,
-            position: ref mut right_position,
-        } = right;
+            let (left_array_iter, left_position) = left.iter_position();
+            let (right_array_iter, right_position) = right.iter_position();
 
-        FromIterator::from_iter(left_array.iter().zip(right_array.iter()).map(|(l, r)| {
-            let left_value = unsafe { ptr::read(l) };
-            let right_value = unsafe { ptr::read(r) };
+            FromIterator::from_iter(left_array_iter.zip(right_array_iter).map(|(l, r)| {
+                let left_value = ptr::read(l);
+                let right_value = ptr::read(r);
 
-            *left_position += 1;
-            *right_position += 1;
+                *left_position += 1;
+                *right_position += 1;
 
-            f(left_value, right_value)
-        }))
+                f(left_value, right_value)
+            }))
+        }
     }
 
     #[doc(hidden)]
@@ -345,24 +394,23 @@ where
         Self::Length: ArrayLength<B> + ArrayLength<U>,
         F: FnMut(Lhs::Item, Self::Item) -> U,
     {
-        let mut right = ArrayConsumer::new(self);
+        unsafe {
+            let mut right = ArrayConsumer::new(self);
 
-        let ArrayConsumer {
-            array: ref right_array,
-            position: ref mut right_position,
-        } = right;
+            let (right_array_iter, right_position) = right.iter_position();
 
-        FromIterator::from_iter(
-            lhs.into_iter()
-                .zip(right_array.iter())
-                .map(|(left_value, r)| {
-                    let right_value = unsafe { ptr::read(r) };
+            FromIterator::from_iter(
+                lhs.into_iter()
+                    .zip(right_array_iter)
+                    .map(|(left_value, r)| {
+                        let right_value = ptr::read(r);
 
-                    *right_position += 1;
+                        *right_position += 1;
 
-                    f(left_value, right_value)
-                }),
-        )
+                        f(left_value, right_value)
+                    }),
+            )
+        }
     }
 }
 
@@ -385,20 +433,19 @@ where
         Self: MappedGenericSequence<T, U>,
         F: FnMut(T) -> U,
     {
-        let mut source = ArrayConsumer::new(self);
+        unsafe {
+            let mut source = ArrayConsumer::new(self);
 
-        let ArrayConsumer {
-            ref array,
-            ref mut position,
-        } = source;
+            let (array_iter, position) = source.iter_position();
 
-        FromIterator::from_iter(array.iter().map(|src| {
-            let value = unsafe { ptr::read(src) };
+            FromIterator::from_iter(array_iter.map(|src| {
+                let value = ptr::read(src);
 
-            *position += 1;
+                *position += 1;
 
-            f(value)
-        }))
+                f(value)
+            }))
+        }
     }
 
     #[inline]
@@ -417,20 +464,19 @@ where
     where
         F: FnMut(U, T) -> U,
     {
-        let mut source = ArrayConsumer::new(self);
+        unsafe {
+            let mut source = ArrayConsumer::new(self);
 
-        let ArrayConsumer {
-            ref array,
-            ref mut position,
-        } = source;
+            let (array_iter, position) = source.iter_position();
 
-        array.iter().fold(init, |acc, src| {
-            let value = unsafe { ptr::read(src) };
+            array_iter.fold(init, |acc, src| {
+                let value = ptr::read(src);
 
-            *position += 1;
+                *position += 1;
 
-            f(acc, value)
-        })
+                f(acc, value)
+            })
+        }
     }
 }
 
@@ -520,17 +566,21 @@ where
         let iter = iter.into_iter();
 
         if iter.len() == N::to_usize() {
-            let mut destination = ArrayBuilder::new();
+            unsafe {
+                let mut destination = ArrayBuilder::new();
 
-            for (dst, src) in destination.array.iter_mut().zip(iter.into_iter()) {
-                unsafe {
-                    ptr::write(dst, src);
+                {
+                    let (destination_iter, position) = destination.iter_position();
+
+                    destination_iter.zip(iter).for_each(|(dst, src)| {
+                        ptr::write(dst, src);
+
+                        *position += 1;
+                    });
                 }
 
-                destination.position += 1;
+                Some(destination.into_inner())
             }
-
-            Some(destination.into_inner())
         } else {
             None
         }
@@ -566,7 +616,7 @@ mod test {
 
     #[test]
     fn test_assembly() {
-        use functional::*;
+        use crate::functional::*;
 
         let a = black_box(arr![i32; 1, 3, 5, 7]);
         let b = black_box(arr![i32; 2, 4, 6, 8]);
